@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -211,8 +211,8 @@ public:
    void Connect(const char *h,const char *p);
    void ConnectVerify();
    void PathVerify(const Path &);
-   virtual void AnonymousLogin();
    virtual void Login(const char *u,const char *p);
+   void AnonymousLogin() { Login(0,0); }
 
    // reset location-related state on Connect/Login/AnonymousLogin
    virtual void ResetLocationData();
