@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -115,17 +115,16 @@ FileSet *Ftp::ParseLongList(const char *buf,int len,int *err_ret) const
 	       best_err1=&err[i];
 	    if(*best_err2>err[i] && best_err1!=&err[i])
 	       best_err2=&err[i];
-	    if(*best_err2 > (*best_err1+1)*16)
-	    {
-	       i=best_err1-err;
-	       guessed_parser=line_parsers[i];
-	       the_set=&set[i];
-	       the_err=&err[i];
-	       break;
-	    }
 	    if(*best_err1>16)
 	       goto leave; // too many errors with best parser.
 	 }
+	 if(*best_err2 > (*best_err1+1)*16)
+	 {
+	    i=best_err1-err;
+	    guessed_parser=line_parsers[i];
+	    the_set=&set[i];
+	    the_err=&err[i];
+	 }
       }
       else
       {
@@ -810,7 +809,7 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
 	 continue;
       }
    }
-   if(name==0 || !type_known)
+   if(name==0 || !*name || !type_known)
       ERR;
 
    fi=new FileInfo(name);
@@ -835,7 +834,7 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
    return fi;
 }
 
-Ftp::FtpLineParser Ftp::line_parsers[number_of_parsers+1]={
+Ftp::FtpLineParser Ftp::line_parsers[number_of_parsers]={
    ParseFtpLongList_UNIX,
    ParseFtpLongList_NT,
    ParseFtpLongList_EPLF,
@@ -843,5 +842,4 @@ Ftp::FtpLineParser Ftp::line_parsers[number_of_parsers+1]={
    ParseFtpLongList_AS400,
    ParseFtpLongList_OS2,
    ParseFtpLongList_MacWebStar,
-   0
 };
