@@ -710,6 +710,22 @@ void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
 	 no_target_dir=true;
 	 return;
       }
+      if(session==source_session && create_target_dir
+      && !FlagSet(NO_EMPTY_DIRS) && !skip_noaccess)
+      {
+	 // create target dir even if failed to cd to source dir.
+	 if(script)
+	    fprintf(script,"mkdir %s\n",target_session->GetFileURL(target_dir));
+	 if(!script_only)
+	 {
+	    ArgV *a=new ArgV("mkdir");
+	    a->Append(target_dir);
+	    mkdirJob *mkj=new mkdirJob(target_session->Clone(),a);
+	    mkj->cmdline.set_allocated(a->Combine());
+	    JobStarted(mkj);
+	 }
+      }
+      remove_this_source_dir=false;
       eprintf("mirror: %s\n",session->StrError(res));
       stats.error_count++;
       assert(transfer_count>=root_transfer_count);
@@ -753,7 +769,7 @@ void MirrorJob::HandleListInfoCreation(const FileAccessRef& session,SMTaskRef<Li
    if(flags&RETR_SYMLINKS)
       list_info->FollowSymlinks();
 
-   list_info->SetExclude(relative_dir,exclude);
+   list_info->SetExclude(relative_dir,top_exclude?top_exclude:exclude);
    list_info->Roll();
 }
 
@@ -789,11 +805,14 @@ int   MirrorJob::Do()
    switch(state)
    {
    case(INITIAL_STATE):
+      remove_this_source_dir=(remove_source_dirs && source_dir.last_char()!='/');
+      if(!strcmp(target_dir,".") || !strcmp(target_dir,"..") || (FlagSet(SCAN_ALL_FIRST) && parent_mirror))
+	 create_target_dir=false;
+
       source_session->Chdir(source_dir);
       source_redirections=0;
       source_session->Roll();
       set_state(CHANGING_DIR_SOURCE);
-      remove_this_source_dir=(remove_source_dirs && source_dir.last_char()!='/');
       m=MOVED;
       /*fallthrough*/
    case(CHANGING_DIR_SOURCE):
@@ -807,8 +826,6 @@ int   MirrorJob::Do()
 
    pre_MAKE_TARGET_DIR:
    {
-      if(!strcmp(target_dir,".") || !strcmp(target_dir,"..") || (FlagSet(SCAN_ALL_FIRST) && parent_mirror))
-	 create_target_dir=false;
       if(!create_target_dir)
 	 goto pre_CHANGING_DIR_TARGET;
       if(target_is_local)
@@ -1722,6 +1739,7 @@ CMD(mirror)
       {"verbose",optional_argument,0,'v'},
       {"newer-than",required_argument,0,'N'},
       {"file",required_argument,0,'f'},
+      {"directory",required_argument,0,'F'},
       {"older-than",required_argument,0,OPT_OLDER_THAN},
       {"size-range",required_argument,0,OPT_SIZE_RANGE},
       {"dereference",no_argument,0,'L'},
@@ -1784,8 +1802,11 @@ CMD(mirror)
    const char *script_file=0;
    const char *on_change=0;
    const char *recursion_mode=0;
+   bool single_file=false;
+   bool single_dir=false;
 
    Ref<PatternSet> exclude;
+   Ref<PatternSet> top_exclude;
 
    if(!ResMgr::QueryBool("mirror:set-permissions",0))
       flags|=MirrorJob::NO_PERMS;
@@ -1798,7 +1819,7 @@ CMD(mirror)
    const char *target_dir=NULL;
 
    args->rewind();
-   while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LP:af:O:",mirror_opts,0))!=EOF)
+   while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LP:af:F:O:",mirror_opts,0))!=EOF)
    {
       switch(opt)
       {
@@ -1875,13 +1896,27 @@ CMD(mirror)
       case('N'):
 	 newer_than=optarg;
 	 break;
-      case('f'):
+      case('f'): // mirror for a single file (or glob pattern).
+	 single_file=true;
+	 /*fallthrough*/
+      case('F'): // mirror for a single directory (or glob pattern).
       {
-	 // mirror for a single file (or glob pattern).
-	 recursion_mode="never";
-	 MirrorJob::AddPattern(exclude,'I',basename_ptr(optarg));
-	 source_dir=dirname(optarg);
-	 source_dir=alloca_strdup(source_dir); // save the temp string
+	 xstring pattern(basename_ptr(optarg));
+	 if(opt=='F') {
+	    single_dir=true;
+	    if(pattern.last_char()!='/')
+	       pattern.append('/');
+	 }
+	 if(!top_exclude)
+	    top_exclude=new PatternSet();
+	 top_exclude->Add(PatternSet::INCLUDE,new PatternSet::Glob(pattern));
+	 const char *dir=dirname(optarg);
+	 if(source_dir && strcmp(source_dir,dir)) {
+	    eprintf(_("%s: multiple --file or --directory options must have the same base directory\n"),args->a0());
+	    goto no_job;
+	 }
+	 if(!source_dir)
+	    source_dir=alloca_strdup(dir); // save the temp string
 	 break;
       }
       case('O'):
@@ -1962,7 +1997,6 @@ CMD(mirror)
 	 break;
       case(OPT_NO_EMPTY_DIRS):
 	 no_empty_dirs=true;
-	 flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
 	 break;
       case(OPT_DEPTH_FIRST):
 	 flags|=MirrorJob::DEPTH_FIRST;
@@ -2102,10 +2136,18 @@ CMD(mirror)
    if(no_empty_dirs)
       flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
 
+   if(recursion_mode && strcasecmp(recursion_mode,"always")
+   && (flags&MirrorJob::DEPTH_FIRST)) {
+      eprintf("%s: --recursion-mode=%s conflicts with other specified options\n",
+	 args->a0(),recursion_mode);
+      return 0;
+   }
+
    JobRef<MirrorJob> j(new MirrorJob(0,source_session.borrow(),target_session.borrow(),source_dir,target_dir));
    j->SetFlags(flags,1);
    j->SetVerbose(verbose);
    j->SetExclude(exclude.borrow());
+   j->SetTopExclude(top_exclude.borrow());
 
    if(newer_than)
       j->SetNewerThan(newer_than);
@@ -2129,6 +2171,9 @@ CMD(mirror)
    if(use_pget>1 && !(flags&MirrorJob::ASCII))
       j->SetPGet(use_pget);
 
+   if(!recursion_mode && single_file && !single_dir)
+      recursion_mode="never";
+
    if(recursion_mode) {
       const char *err=j->SetRecursionMode(recursion_mode);
       if(err) {
@@ -2168,33 +2213,16 @@ CDECL void module_init()
 {
    CmdExec::RegisterCommand("mirror",cmd_mirror,0,
       N_("\n"
-	 "Mirror specified remote directory to local directory\n\n"
-	 " -c, --continue         continue a mirror job if possible\n"
-	 " -e, --delete           delete files not present at remote site\n"
-	 "     --delete-first     delete old files before transferring new ones\n"
-	 " -s, --allow-suid       set suid/sgid bits according to remote site\n"
-	 "     --allow-chown      try to set owner and group on files\n"
-	 "     --ignore-time      ignore time when deciding whether to download\n"
-	 " -n, --only-newer       download only newer files (-c won't work)\n"
-	 " -r, --no-recursion     don't go to subdirectories\n"
-	 " -p, --no-perms         don't set file permissions\n"
-	 "     --no-umask         don't apply umask to file modes\n"
+	 "Mirror specified remote directory to local directory\n"
+	 "\n"
 	 " -R, --reverse          reverse mirror (put files)\n"
-	 " -L, --dereference      download symbolic links as files\n"
-	 " -N, --newer-than=SPEC  download only files newer than specified time\n"
-	 " -P, --parallel[=N]     download N files in parallel\n"
-	 " -i RX, --include RX    include matching files\n"
-	 " -x RX, --exclude RX    exclude matching files\n"
-	 "                        RX is extended regular expression\n"
-	 " -v, --verbose[=N]      verbose operation\n"
-	 "     --log=FILE         write lftp commands being executed to FILE\n"
-	 "     --script=FILE      write lftp commands to FILE, but don't execute them\n"
-	 "     --just-print, --dry-run    same as --script=-\n"
+	 "Lots of other options are documented in the man page lftp(1).\n"
 	 "\n"
 	 "When using -R, the first directory is local and the second is remote.\n"
-	 "If the second directory is omitted, basename of first directory is used.\n"
+	 "If the second directory is omitted, basename of the first directory is used.\n"
 	 "If both directories are omitted, current local and remote directories are used.\n"
-	 "See lftp(1) for a complete list of options.\n"
+	 "\n"
+	 "See the man page lftp(1) for a complete documentation.\n"
       )
    );
 }
