@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 2009 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,17 +14,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ProtoLog.h,v 1.1 2009/07/17 12:50:46 lav Exp $ */
-
 #ifndef PROTOLOG_H
 #define PROTOLOG_H
 
+#include "xstring.h"
+
 class ProtoLog
 {
+   static bool WillOutput(int level);
 public:
    static void Log2(int level,xstring& str);
    static void Log3(int level,const char *prefix,const char *str);
