@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ CDECL_BEGIN
 
 #define USE_VARARGS 1
 #define PREFER_STDARG 1
-#include "readline/readline.h"
+#include <readline/readline.h>
 CDECL_END
 
 #ifndef GLOB_PERIOD
@@ -66,6 +66,7 @@ static bool inhibit_tilde;
 
 static bool shell_cmd;
 static bool quote_glob;
+static bool quote_glob_basename;
 
 char *command_generator(const char *text,int state)
 {
@@ -384,6 +385,9 @@ static completion_type cmd_completion_type(const char *cmd,int start)
    || !strcmp(buf,"mrm"))
       quote_glob=true;
 
+   if(!strcmp(buf,"cls"))
+      quote_glob_basename=true;
+
    if(!strcmp(buf,"cd")
    || !strcmp(buf,"mkdir"))
       return REMOTE_DIR; /* append slash automatically */
@@ -619,6 +623,7 @@ static char **lftp_completion (const char *text,int start,int end)
    rl_ignore_some_completions_function=0;
    shell_cmd=false;
    quote_glob=false;
+   quote_glob_basename=false;
    inhibit_tilde=false;
    delete glob_res;
    glob_res=0;
@@ -763,7 +768,6 @@ static char **lftp_completion (const char *text,int start,int end)
 
       if(rg)
       {
-	 rg->NoMatchPeriod();
 	 rg->NoInhibitTilde();
 	 if(fso.patterns_casefold) {
 	    rl_variable_bind("completion-ignore-case", "1");
@@ -943,6 +947,7 @@ backslash_quote (char *string)
 {
   int c;
   char *result, *r, *s;
+  char *bn = basename_ptr(string);
 
   result = (char*)xmalloc (2 * strlen (string) + 1);
 
@@ -950,15 +955,16 @@ backslash_quote (char *string)
     {
       switch (c)
 	{
- 	case '(': case ')':
- 	case '{': case '}':			/* reserved words */
- 	case '^':
- 	case '$': case '`':			/* expansion chars */
+	case '(': case ')':
+	case '{': case '}':			/* reserved words */
+	case '^':
+	case '$': case '`':			/* expansion chars */
 	  if(!shell_cmd)
 	    goto def;
- 	case '*': case '[': case '?': case ']':	/* globbing chars */
-	  if(!shell_cmd && !quote_glob)
+	case '*': case '[': case '?': case ']':	/* globbing chars */
+	  if(!shell_cmd && !quote_glob && (!quote_glob_basename || s<bn))
 	    goto def;
+	  /*fallthrough*/
 	case ' ': case '\t': case '\n':		/* IFS white space */
 	case '"': case '\'': case '\\':		/* quoting chars */
 	case '|': case '&': case ';':		/* shell metacharacters */
@@ -1234,7 +1240,8 @@ static int lftp_complete_remote(int count,int key)
 int lftp_rl_getc(FILE *file)
 {
    SignalHook::DoCount(SIGINT);
-   CharReader r(fileno(file));
+   SMTaskRef<CharReader> rr(new CharReader(fileno(file)));
+   CharReader& r=*rr.get_non_const();
    for(;;)
    {
       SMTask::Schedule();
