@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ public:
 
    entry *_each_begin();
    entry *_each_next();
-   const xstring& each_key() const { return last_entry->key; }
+   const xstring& each_key() const { return last_entry ? last_entry->key : xstring::null; }
    bool each_finished() const { return last_entry==0; }
 
    int count() const { return entry_count; }
@@ -80,7 +80,7 @@ public:
       entry **e=_lookup(key);
       if(e && *e)
 	 return payload(*e);
-      return zero;
+      return add(key,zero);
    }
    const T& lookup(const xstring& key) const {
       entry *e=_lookup_c(key);
@@ -91,7 +91,7 @@ public:
    const T& lookup(const char *key) const { return lookup(xstring::get_tmp(key)); }
    T& operator[](const xstring& key) { return lookup_Lv(key); }
    void remove(const xstring& key) { _remove(_lookup(key)); }
-   void add(const xstring& key,T e0) { payload(_add(key))=e0; }
+   T& add(const xstring& key,T e0) { return payload(_add(key))=e0; }
    const T& each_begin() { entry *e=_each_begin(); return e?payload(e):zero; }
    const T& each_next()  { entry *e=_each_next();  return e?payload(e):zero; }
    void move_here(xmap<T> &o) { _move_here(o); }
@@ -103,7 +103,7 @@ template<class T> T xmap<T>::zero;
 
 template<class T> class xmap_p : public _xmap
 {
-   virtual void dispose(T *p) { delete p; }
+   void dispose(T *p) { delete p; }
 public:
    xmap_p() : _xmap(sizeof(T*)) {}
    ~xmap_p() {
