@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,10 +62,36 @@ CDECL int vsnprintf(char *,size_t,const char *,va_list);
 CDECL int snprintf(char *,size_t,const char *,...);
 #endif
 
-int xstrcmp(const char *s1,const char *s2);
-int xstrncmp(const char *s1,const char *s2,size_t len);
-int xstrcasecmp(const char *s1,const char *s2);
-size_t xstrlen(const char *s);
+static inline int xstrcmp(const char *s1,const char *s2)
+{
+   if(s1==s2)
+      return 0;
+   if(s1==0 || s2==0)
+      return 1;
+   return strcmp(s1,s2);
+}
+static inline int xstrncmp(const char *s1,const char *s2,size_t len)
+{
+   if(s1==s2 || len==0)
+      return 0;
+   if(s1==0 || s2==0)
+      return 1;
+   return strncmp(s1,s2,len);
+}
+static inline int xstrcasecmp(const char *s1,const char *s2)
+{
+   if(s1==s2)
+      return 0;
+   if(s1==0 || s2==0)
+      return 1;
+   return strcasecmp(s1,s2);
+}
+static inline size_t xstrlen(const char *s)
+{
+   if(s==0)
+      return 0;
+   return strlen(s);
+}
 
 #include <stdarg.h>
 #include "xmalloc.h"
@@ -73,7 +99,7 @@ size_t xstrlen(const char *s);
 /* this is a small and fast dynamic string class */
 /* mostly used as xstrdup/xfree replacement */
 
-enum { URL_DECODE_PLUS=1 };
+enum { URL_DECODE_PLUS=1, URL_ALLOW_8BIT=2 };
 
 class xstring0 // base class
 {
@@ -108,19 +134,25 @@ public:
    const char *set(const char *s) { return xstrset(buf,s); }
    const char *nset(const char *s,int n) { return xstrset(buf,s,n); }
    const char *set_allocated(char *s) { xfree(buf); return buf=s; }
+   const char *move_here(xstring_c& s) { return set_allocated(s.borrow()); }
    const char *vset(...) ATTRIBUTE_SENTINEL;
    void truncate(size_t n=0) { if(buf) buf[n]=0; }
    char *borrow() { return replace_value(buf,(char*)0); }
    bool begins_with(const char *s) const { return !strncmp(buf,s,strlen(s)); };
-   bool eq(const char *s) { return !xstrcmp(buf,s); }
-   bool ne(const char *s) { return !eq(s); }
+   bool prefixes(const char *s) const { return !strncmp(buf,s,length()); };
+   bool eq(const char *s) const { return !xstrcmp(buf,s); }
+   bool ne(const char *s) const { return !eq(s); }
+   bool eq_nc(const char *s) const { return !xstrcasecmp(buf,s); }
    size_t length() const { return xstrlen(buf); }
    void set_length(size_t n) { if(buf) buf[n]=0; }
+   char last_char() const { size_t len=length(); return len>0?buf[len-1]:0; }
 
    void unset() { xfree(buf); buf=0; }
    void _set(const char *s) { buf=xstrdup(s); }
 
    xstring_c& url_decode(int flags=0);
+
+   static const xstring_c null;
 };
 class xstring_ca : public xstring_c
 {
@@ -174,6 +206,8 @@ public:
    xstring& set_substr(int start,size_t sublen,const char *,size_t);
    xstring& set_substr(int start,size_t sublen,const char *);
    xstring& set_substr(int start,size_t sublen,const xstring &s) { return set_substr(start,sublen,s.get(),s.length()); }
+   xstring& prepend(const char *s,size_t len) { return set_substr(0,0,s,len); }
+   xstring& prepend(const xstring &s) { return prepend(s.get(),s.length()); }
 
    xstring& append(const char *s);
    xstring& append(char c);
@@ -196,7 +230,8 @@ public:
    static xstring& cat(const char *first,...) ATTRIBUTE_SENTINEL;
    static xstring& join(const char *sep,int n,...);
 
-   void truncate(size_t n=0);
+   void truncate() { set_length(0); }
+   void truncate(size_t n);
    void truncate_at(char c);
    /* set_length can be used to extend the string, e.g. after modification
       with get_space+get_non_const. */
@@ -207,10 +242,16 @@ public:
    bool begins_with(const char *s) const { return begins_with(s,strlen(s)); };
    bool ends_with(const char *o_buf,size_t o_len) const;
    bool ends_with(const char *s) const { return ends_with(s,strlen(s)); };
+   bool prefixes(const char *s) const { return !strncmp(buf,s,length()); };
    bool eq(const char *o_buf,size_t o_len) const;
    bool eq(const char *s) const { return eq(s,strlen(s)); }
    bool eq(const xstring&o) const { return eq(o.get(),o.length()); }
    bool ne(const xstring&o) const { return !eq(o); }
+   int cmp(const char *o_buf,size_t o_len) const;
+   int cmp(const xstring&o) const { return cmp(o.get(),o.length()); }
+   bool eq_nc(const char *o_buf,size_t o_len) const;
+   bool eq_nc(const char *s) const { return eq_nc(s,strlen(s)); }
+   bool eq_nc(const xstring&o) const { return eq_nc(o.get(),o.length()); }
    bool chomp(char c='\n');
    void rtrim(char c=' ');
    char last_char() const { return len>0?buf[len-1]:0; }
@@ -228,16 +269,19 @@ public:
    const char *hexdump() const;
 
    xstring& url_decode(int flags=0);
-   xstring& append_url_encoded(const char *s,int len,const char *unsafe);
-   xstring& append_url_encoded(const char *s,const char *unsafe) { return append_url_encoded(s,strlen(s),unsafe); }
-   xstring& append_url_encoded(const xstring& s,const char *unsafe) { return append_url_encoded(s,s.length(),unsafe); }
+   xstring& append_url_encoded(const char *s,int len,const char *unsafe,unsigned flags=0);
+   xstring& append_url_encoded(const char *s,const char *unsafe,unsigned flags=0) { return append_url_encoded(s,strlen(s),unsafe,flags); }
+   xstring& append_url_encoded(const xstring& s,const char *unsafe,unsigned flags=0) { return append_url_encoded(s,s.length(),unsafe,flags); }
 
    xstring& append_quoted(const char *s,int len);
    xstring& append_quoted(const char *s) { return append_quoted(s,strlen(s)); }
 
    xstring& hex_decode();
 
-   static xstring null;
+   xstring& c_lc();
+   xstring& c_ucfirst();
+
+   static const xstring null;
 };
 class xstring_clonable : public xstring {};
 
